import subprocess
import os
import tkinter as tk
from tkinter import ttk, messagebox, filedialog
import ttkbootstrap as ttk
from ttkbootstrap.constants import *
import re
import threading

class YouTubeDownloaderApp:
    def __init__(self, root):
        self.root = root
        self.root.title("מוריד סרטוני יוטיוב")
        self.root.geometry("600x650")
        
        # הגדרת ערכת נושא מודרנית
        style = ttk.Style("darkly")
        
        # יצירת מיכל ראשי
        self.create_main_container()
        
        # הגדרת תיקיית ברירת מחדל להורדות
        default_path = os.path.join(os.path.expanduser("~"), "Downloads")
        self.output_path.insert(0, default_path)

    def create_main_container(self):
        main_container = ttk.Frame(self.root, padding="20")
        main_container.pack(fill="both", expand=True)
        
        # כותרת ראשית
        self.create_header(main_container)
        
        # אזור URL
        self.create_url_section(main_container)
        
        # אזור בחירת תיקייה
        self.create_path_section(main_container)
        
        # אפשרויות הורדה
        self.create_options_section(main_container)
        
        # כפתור הורדה ופס התקדמות
        self.create_download_section(main_container)
        
        # אזור לוג
        self.create_log_section(main_container)

    def create_header(self, container):
        header = ttk.Label(
            container,
            text="הורדת סרטונים מיוטיוב",
            font=("Segoe UI", 24, "bold"),
            bootstyle="inverse-primary"
        )
        header.pack(pady=(0, 20))

    def create_url_section(self, container):
        url_frame = ttk.LabelFrame(container, text="קישור", padding="10")
        url_frame.pack(fill="x", pady=(0, 10))
        
        self.url_entry = ttk.Entry(
            url_frame,
            width=60,
            font=("Segoe UI", 10),
            bootstyle="primary"
        )
        self.url_entry.pack(fill="x", pady=(0, 5))
        
        buttons_frame = ttk.Frame(url_frame)
        buttons_frame.pack(fill="x")
        
        copy_button = ttk.Button(
            buttons_frame,
            text="העתק",
            command=self.copy_url,
            bootstyle="info-outline",
            width=15
        )
        copy_button.pack(side="right", padx=5)
        
        paste_button = ttk.Button(
            buttons_frame,
            text="הדבק",
            command=self.paste_url,
            bootstyle="info-outline",
            width=15
        )
        paste_button.pack(side="right")

    def create_path_section(self, container):
        path_frame = ttk.LabelFrame(container, text="תיקיית שמירה", padding="10")
        path_frame.pack(fill="x", pady=10)
        
        self.output_path = ttk.Entry(
            path_frame,
            font=("Segoe UI", 10),
            bootstyle="primary"
        )
        self.output_path.pack(side="left", fill="x", expand=True, padx=(0, 5))
        
        browse_button = ttk.Button(
            path_frame,
            text="בחר תיקייה",
            command=self.select_directory,
            bootstyle="secondary-outline",
            width=15
        )
        browse_button.pack(side="right")

    def create_options_section(self, container):
        options_frame = ttk.LabelFrame(container, text="אפשרויות הורדה", padding="10")
        options_frame.pack(fill="x", pady=10)
        
        # סוג הורדה
        format_frame = ttk.Frame(options_frame)
        format_frame.pack(fill="x", pady=(0, 10))
        
        ttk.Label(
            format_frame, 
            text="סוג הורדה:",
            font=("Segoe UI", 10, "bold")
        ).pack(side="right", padx=5)
        
        self.format_var = tk.StringVar(value="video")
        ttk.Radiobutton(
            format_frame,
            text="וידאו",
            variable=self.format_var,
            value="video",
            bootstyle="primary-toolbutton"
        ).pack(side="right", padx=5)
        
        ttk.Radiobutton(
            format_frame,
            text="אודיו בלבד",
            variable=self.format_var,
            value="audio",
            bootstyle="primary-toolbutton"
        ).pack(side="right", padx=5)
        
        # סוג תוכן
        content_frame = ttk.Frame(options_frame)
        content_frame.pack(fill="x")
        
        ttk.Label(
            content_frame,
            text="סוג תוכן:",
            font=("Segoe UI", 10, "bold")
        ).pack(side="right", padx=5)
        
        self.content_type = tk.StringVar(value="single")
        for text, value in [
            ("סרטון בודד", "single"),
            ("פלייליסט", "playlist"),
            ("ערוץ", "channel")
        ]:
            ttk.Radiobutton(
                content_frame,
                text=text,
                variable=self.content_type,
                value=value,
                bootstyle="primary-toolbutton"
            ).pack(side="right", padx=5)
        
        # הוספת אפשרויות פרוקסי
        proxy_frame = ttk.Frame(options_frame)
        proxy_frame.pack(fill="x", pady=(10, 0))
        
        ttk.Label(
            proxy_frame,
            text="פרוקסי:",
            font=("Segoe UI", 10, "bold")
        ).pack(side="right", padx=5)
        
        self.proxy_var = tk.StringVar(value="system")
        
        for text, value in [
            ("הגדרות מערכת", "system"),
            ("ללא פרוקסי", "none"),
            ("פרוקסי מותאם", "custom")
        ]:
            ttk.Radiobutton(
                proxy_frame,
                text=text,
                variable=self.proxy_var,
                value=value,
                bootstyle="primary-toolbutton",
                command=self.show_proxy_dialog if value == "custom" else None
            ).pack(side="right", padx=5)

    def create_download_section(self, container):
        download_button = ttk.Button(
            container,
            text="התחל הורדה",
            command=self.download_video,
            bootstyle="success",
            width=20
        )
        download_button.pack(pady=20)
        
        self.progress = ttk.Progressbar(
            container,
            bootstyle="success-striped",
            mode="determinate",
            length=550
        )
        self.progress.pack(fill="x", pady=(0, 10))
        
        self.status_var = tk.StringVar(value="מוכן להורדה")
        status_label = ttk.Label(
            container,
            textvariable=self.status_var,
            font=("Segoe UI", 10),
            bootstyle="primary"
        )
        status_label.pack()

    def create_log_section(self, container):
        log_frame = ttk.LabelFrame(container, text="לוג פעילות", padding="10")
        log_frame.pack(fill="both", expand=True, pady=(10, 0))
        
        self.log_text = tk.Text(
            log_frame,
            height=8,
            font=("Consolas", 9),
            wrap="word",
            bg="#2c2c2c",
            fg="#ffffff"
        )
        scrollbar = ttk.Scrollbar(log_frame, bootstyle="primary-round")
        scrollbar.config(command=self.log_text.yview)
        self.log_text.config(yscrollcommand=scrollbar.set)
        self.log_text.pack(side="left", fill="both", expand=True)
        scrollbar.pack(side="right", fill="y")
        
    def select_directory(self):
        directory = filedialog.askdirectory()
        if directory:
            self.output_path.delete(0, tk.END)
            self.output_path.insert(0, directory)

    def copy_url(self):
        url = self.url_entry.get().strip()
        if url:
            self.root.clipboard_clear()
            self.root.clipboard_append(url)
            self.status_var.set("הקישור הועתק")
        else:
            self.status_var.set("אין קישור להעתקה")

    def paste_url(self):
        try:
            clipboard_text = self.root.clipboard_get()
            self.url_entry.delete(0, tk.END)
            self.url_entry.insert(0, clipboard_text)
            self.status_var.set("הקישור הודבק")
        except:
            self.status_var.set("אין טקסט בלוח ההעתקה")

    def log_message(self, message):
        self.root.after(0, lambda: self.log_text.insert(tk.END, f"{message}\n"))
        self.root.after(0, lambda: self.log_text.see(tk.END))

    def show_proxy_dialog(self):
        dialog = tk.Toplevel(self.root)
        dialog.title("הגדרות פרוקסי")
        dialog.geometry("400x150")
        dialog.resizable(False, False)
        
        frame = ttk.Frame(dialog, padding="20")
        frame.pack(fill="both", expand=True)
        
        ttk.Label(
            frame,
            text="כתובת פרוקסי:",
            font=("Segoe UI", 10)
        ).pack(anchor="e")
            
        proxy_entry = ttk.Entry(frame, width=40)
        proxy_entry.pack(pady=(5, 20))
        proxy_entry.insert(0, self.custom_proxy if hasattr(self, 'custom_proxy') else "")
        
        ttk.Label(
            frame,
            text="פורמט: protocol://host:port או host:port",
            font=("Segoe UI", 8)
        ).pack()
        
        def save_proxy():
            self.custom_proxy = proxy_entry.get().strip()
            dialog.destroy()
        
        ttk.Button(
            frame,
            text="שמור",
            command=save_proxy,
            bootstyle="success"
        ).pack(pady=(10, 0))

    def get_proxy_settings(self):
        proxy_type = self.proxy_var.get()
        
        if proxy_type == "none":
            return ""
        elif proxy_type == "custom":
            return getattr(self, 'custom_proxy', '')
        else:  # system
            try:
                import winreg
                with winreg.OpenKey(
                    winreg.HKEY_CURRENT_USER,
                    r"Software\Microsoft\Windows\CurrentVersion\Internet Settings"
                ) as key:
                    proxy_enable = winreg.QueryValueEx(key, "ProxyEnable")[0]
                    if proxy_enable:
                        return winreg.QueryValueEx(key, "ProxyServer")[0]
            except:
                pass
            return ""

    def download_video(self):
        url = self.url_entry.get().strip()
        output_path = self.output_path.get().strip()
        
        # בדיקות תקינות
        if not url:
            messagebox.showerror("שגיאה", "יש להזין קישור יוטיוב תקין")
            return
        if not output_path:
            messagebox.showerror("שגיאה", "יש לבחור תיקיית שמירה")
            return
        
        # יצירת תיקיית היעד אם לא קיימת
        if not os.path.exists(output_path):
            try:
                os.makedirs(output_path)
            except:
                messagebox.showerror("שגיאה", "לא ניתן ליצור את תיקיית היעד")
                return
        
        # הכנת פקודה בסיסית
        yt_dlp_path = os.path.join(os.path.dirname(__file__), "yt-dlp.exe")
        command = [
            yt_dlp_path,
            "--no-check-certificate",
            "--force-ipv4",
            "--geo-bypass",
            "--no-part",
            "--no-mtime"
        ]
        
        # טיפול בסוג תוכן
        content_type = self.content_type.get()
        if content_type in ["playlist", "channel"]:
            command.extend(["--yes-playlist"])
        else:
            command.extend(["--no-playlist"])
        
        # קביעת קובץ יעד
        if content_type in ["playlist", "channel"]:
            output_template = os.path.join(output_path, "%(playlist_title)s", "%(title)s.%(ext)s")
        else:
            output_template = os.path.join(output_path, "%(title)s.%(ext)s")
        command.extend(["-o", output_template])
        
        # בחירת פורמט הורדה
        if self.format_var.get() == "audio":
            command.extend([
                "-f", "ba",
                "--extract-audio",
                "--audio-format", "mp3",
                "--audio-quality", "0"
            ])
        else:
            command.extend([
                "-f", "bv*+ba/b"
            ])
        
        # הגדרות פרוקסי
        proxy = self.get_proxy_settings()
        if proxy:
            if not re.match(r'^\w+://', proxy):
                proxy = f"http://{proxy}"
            command.extend(["--proxy", proxy])
        
        # הוספת ה-URL בסוף הפקודה
        command.append(url)
        
        # הפעלת תהליך ההורדה ברקע
        self.status_var.set("מוריד...")
        self.progress.start()
        
        def run_download():
            try:
                process = subprocess.Popen(
                    command,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    universal_newlines=True,
                    encoding='cp1255'
                )
                while True:
                    line = process.stdout.readline()
                    if not line:
                        break
                    self.log_message(line.strip())
                    if "[download]" in line and "%" in line:
                        try:
                            percent = line.split()[1].replace('%','')
                            self.status_var.set(f"מוריד... {percent}%")
                        except:
                            pass
                    elif "Destination:" in line:
                        filename = line.split("Destination:", 1)[1].strip()
                        self.log_message(f"מוריד: {filename}")
                stdout, stderr = process.communicate()
                if process.returncode == 0:
                    self.root.after(0, lambda: messagebox.showinfo("הצלחה", "ההורדה הושלמה בהצלחה"))
                    self.root.after(0, lambda: self.status_var.set("ההורדה הושלמה בהצלחה!"))
                    self.log_message("ההורדה הושלמה בהצלחה!")
                else:
                    error_msg = stderr if stderr else "שגיאה לא ידועה"
                    self.root.after(0, lambda: messagebox.showerror("שגיאה בהורדה", error_msg))
                    self.root.after(0, lambda: self.status_var.set(f"שגיאה: {error_msg[:50]}..."))
                    self.log_message(f"שגיאה: {error_msg}")
            except Exception as e:
                self.root.after(0, lambda: messagebox.showerror("שגיאה", str(e)))
                self.root.after(0, lambda: self.status_var.set(f"שגיאה: {str(e)}"))
                self.log_message(f"שגיאה: {str(e)}")
            finally:
                self.root.after(0, self.progress.stop)
        
        threading.Thread(target=run_download, daemon=True).start()

if __name__ == "__main__":
    root = tk.Tk()
    app = YouTubeDownloaderApp(root)
    root.mainloop()